#!/usr/bin/perl -w
#
# Copyright (C) 2008 Patrick Fuerlinger
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# History:
# 2008-08-10 pf initial
# 2008-08-31 pf add weekday
#
#//////////////////////////////////////////////////////////////////////////////
use FindBin qw ( $Bin );
use lib "$Bin/../lib/";
use strict;
use warnings;
use ReelBox::Base qw( vdr_status get_config read_file svdrp );
use Storable;
use File::stat;
use Getopt::Long;
use LockFile::Simple qw(lock trylock unlock);
use File::Find;
use File::Basename;
use URI::Escape;
use Time::Local;
use Date::Calc qw(Add_Delta_Days Day_of_Week);
use Proc::PID::File;

my $g_Debug;
my %g_RbcConf       = %{ get_config('/opt/rbc/htdocs/config/reelbox.cfg') };
my $g_RbcStatusHash = "/opt/rbc/var/rbc.hash";
my $g_RbcEpgHash    = "/opt/rbc/var/epg.hash";
my %g_RbcStatus;
my %g_Config;

my %g_WeekDay = (
  '7' => 'Sunday',
  '1' => 'Monday',
  '2' => 'Tuesday',
  '3' => 'Wednesday',
  '4' => 'Thursday',
  '5' => 'Friday',
  '6' => 'Saturday'
);

#//////////////////////////////////////////////////////////////////////////////
# funtions
sub GetArgs {
  my $help;

  GetOptions(
    "--debug"        => \$g_Debug,
    "--housekeeping" => \$g_Config{'housekeeping'},
    "--force"        => \$g_Config{'force'},
    "--help"         => \$help
  );

  &ArgHelp if $help;

}

#//////////////////////////////////////////////////////////////////////////////
sub ArgHelp {

  exit 0;
}

#//////////////////////////////////////////////////////////////////////////////
sub get_epg {
  my $channel;
  my $time;
  my $desc;
  my $text;
  my $fstatus;

  my %NewEPG;
  my @EPG;

  if ( vdr_status ) {
    @EPG = svdrp('LSTE') if vdr_status;
  }
  elsif ( -e '/var/tmp/epg.data' ) {
    @EPG = read_file('/var/tmp/epg.data');
    $fstatus = 1;
  }

  foreach my $ln ( @EPG ) {
    next if ! $ln || $ln eq '';
    chomp($ln);
    $ln =~ s/\r*//;
    $ln =~ s/^215-//;

    next if $ln =~ /^[e|c] /;

    #C S19.2E-1-1093-28453 ARD
    #E 22853 1218385800 19800 60 0
    #T INTERNATIONALE MUSIKFESTSPIELE 2008
    #D description ---
    #X 2 03 deu stereo
    #V 1218385800

    ## channel
    if ( $ln =~ /^C \s*(.+?)\s* (.*)$/ ) {
      print "Channel: $1 -- $2\n" if $g_Debug;
      $channel = $1;

      $time = undef;
      $desc = undef;
      $text = undef;

    }
    ## time
    elsif ( $ln =~ /^E (\d+) (\d+) (\d+?) .*\s*$/ ) {
      $time                                = $2;
      $NewEPG{$channel}{$time}{'channel'}  = $channel;
      $NewEPG{$channel}{$time}{'time_end'} = $2 + $3;
      $NewEPG{$channel}{$time}{'time'}     = $2;

      my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
        localtime($2);

      $sec  = sprintf( "%02d", $sec );
      $min  = sprintf( "%02d", $min );
      $mon  = sprintf( "%02d", $mon + 1 );
      $mday = sprintf( "%02d", $mday );

      $NewEPG{$channel}{$time}{'start_timef'} =
        "$mday.$mon." . ( int($year) + 1900 ) . " $hour:$min";

      $hour = sprintf( "%02d", $hour );

      $NewEPG{$channel}{$time}{'start_time'} = "$hour:$min";
      $NewEPG{$channel}{$time}{'date'} = ( int($year) + 1900 ) . $mon . $mday;
      $NewEPG{$channel}{$time}{'fdate'} =
        ( int($year) + 1900 ) . '-' . $mon . '-' . $mday;
      $NewEPG{$channel}{$time}{'length'} = int( $3 / 60 ) if defined $3;

      ## get weekday
      $NewEPG{$channel}{$time}{'weekday'} =
        $g_WeekDay{ Day_of_Week( $year + 1900, $mon, $mday ) };

      ## calculate end_time
      ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
        localtime( $2 + $3 );

      $sec  = sprintf( "%02d", $sec );
      $min  = sprintf( "%02d", $min );
      $mon  = sprintf( "%02d", $mon + 1 );
      $mday = sprintf( "%02d", $mday );

      $NewEPG{$channel}{$time}{'end_timef'} =
        "$mday.$mon." . ( int($year) + 1900 ) . " $hour:$min";

      $hour = sprintf( "%02d", $hour );

      $NewEPG{$channel}{$time}{'end_time'} = "$hour:$min";

      $desc = undef;
      $text = undef;

      print "TIME: $2\n" if $g_Debug;
    }
    ## Description
    elsif ( $ln =~ /^D (.+)\s*$/ ) {
      next if ( !defined $time );
      $NewEPG{$channel}{$time}{'descr'} = $1;
      print "Desc: $1\n" if $g_Debug;
    }
    ## Text
    elsif ( $ln =~ /^T (.+)\s*$/ ) {
      next if ( !defined $time );
      $NewEPG{$channel}{$time}{'text'} = $1;

      ## get titel subtitel
      if ( $NewEPG{$channel}{$time}{'text'} =~ /^(.*)?\s-\s(.*)$/ ) {
        $NewEPG{$channel}{$time}{'titel'} = $1;
        ## $NewEPG{$channel}{$time}{'subtitel'} = $2;
        $NewEPG{$channel}{$time}{'titel'} = $NewEPG{$channel}{$time}{'text'};
      }
      else {
        $NewEPG{$channel}{$time}{'titel'} = $NewEPG{$channel}{$time}{'text'};
      }

      print "TEXT: $1\n" if $g_Debug;
    }
    elsif ( $ln =~ /^S (.+)\s*$/ ) {
      $NewEPG{$channel}{$time}{'subtitel'} = $1;
    }
    else {
      print "Line: $ln\n" if $g_Debug;
    }
  }

  ## save RbStatus
  store \%NewEPG, $g_RbcEpgHash if vdr_status || $fstatus;
}

## EPG housekeeping ////////////////////////////////////////////////////////////
#  delete old epg entries.
#  update remaining term
sub HouseKeeping {

  ## check rbc hash.
  if ( !-e $g_RbcEpgHash ) {
    print "not yet running. Aborting housekeeping ...\n";
    return undef;
  }

  my %EPG;

  ## get rbc status
  if ( eval { %EPG = %{ retrieve( $g_RbcEpgHash ) } } ) {
    print "load epg hash ok\n" if $g_Debug;
  }
  else {
    unlink($g_RbcEpgHash);
    print "load epg hash failed! Unlink it ...\n" if $g_Debug;
  }

  my $time = timelocal(localtime);

  print "localtime: $time\n" if $g_Debug;

  foreach my $cCount ( sort keys %EPG ) {
    foreach ( sort keys %{ $EPG{$cCount} } ) {

      ## old entrie -> delete
      if ( $time > $EPG{$cCount}{$_}{'time_end'} ) {
        print "delete $cCount $EPG{$cCount}{$_}{'end_time'}\n";
        delete $EPG{$cCount}{$_};
      }
      ## not yet running
      elsif ( $time < $EPG{$cCount}{$_}{'time'} ) {
        $EPG{$cCount}{$_}{'remaining'}     = $EPG{$cCount}{$_}{'length'};
        $EPG{$cCount}{$_}{'remaining_pct'} = 100;
        print "not yet:  $EPG{$cCount}{$_}{'end_time'}\n" if $g_Debug;
      }
      ## running - calculate remaining term
      else {
        $EPG{$cCount}{$_}{'remaining'} =
          int( ( $EPG{$cCount}{$_}{'time_end'} - $time ) / 60 );

        ## calculate percent
        $EPG{$cCount}{$_}{'remaining_pct'} = int(
          100 / $EPG{$cCount}{$_}{'length'} * $EPG{$cCount}{$_}{'remaining'} );

        ## fix zero lenght display: if 0 set 1%
        if ( !defined $EPG{$cCount}{$_}{'remaining_pct'}
          || $EPG{$cCount}{$_}{'remaining_pct'} == 0 )
        {
          $EPG{$cCount}{$_}{'remaining_pct'} = 1;
        }

        print "set remaining: $EPG{$cCount}{$_}{'remaining'}"
          . " pct: $EPG{$cCount}{$_}{'remaining_pct'}\n"
          if $g_Debug;
      }
    }
  }
  ## save RbStatus
  store \%EPG, $g_RbcEpgHash if ( keys %EPG > 0 );

}

#//////////////////////////////////////////////////////////////////////////////
# program starts here ...
sub main {

  &GetArgs;

  ##  check vdrstatus
  #if ( !vdr_status && !$g_Config{'force'} ) {
  #  print "ReelVDR not running! Perform housekeeping ...\n" if $g_Debug;
  #  &HouseKeeping;
  #  exit 0;
  #}

  ## check PID File
  if ( Proc::PID::File->running() ) {
    print "already running ...\n" if $g_Debug;
    exit 0;
  }

  if ( defined $g_Config{'housekeeping'} ) {
    &HouseKeeping;
    exit 0;
  }

  &get_epg;
  &HouseKeeping;

  exit 0;
}

#//////////////////////////////////////////////////////////////////////////////
&main;

exit 1;
